--[[SirRanjid 2020 https://github.com/SirRanjid]]

local files = {}

local lfs = love.filesystem


local function loadluafile(file)
	local ok, chunk, result
	ok, chunk = pcall( lfs.load, file )
	if not ok then
		--MsgC(err_col,"Error loading '"..file.."': "..tostring(chunk) )
	else
		ok, result = pcall(chunk)
	
		if not ok then
			--MsgC(err_col,"Error executing '"..file.."': "..tostring(result) )
		else
			local modtime, errormsg = lfs.getLastModified( file )
			if modtime and not errormsg then
				files[file] = modtime
				--MsgC(suc_col,"Loaded '"..file.."'")
			else
				--MsgC(err_col,"Error loading '"..file.."': "..errormsg)
			end
		end
	end
end

local function recursiveLoad(folder)
	local filesTable = lfs.getDirectoryItems(folder)
	
	local luas, dirs = {}, {}
	
	for i,v in ipairs(filesTable) do
		local file = folder.."/"..v
		
		if lfs.isFile(file) then
			if file:sub(-4,-1) == ".lua" then
				table.insert(luas,file)
			end
		elseif lfs.isDirectory(file) then
			table.insert(dirs,file)
		end
	end
	
	for i,v in ipairs(luas) do --first all files
		loadluafile(v)
	end
	
	for i,v in ipairs(dirs) do --then search deeper
		recursiveLoad(v)
	end
end

GAME.Fonts = {}
do	--put it on another thread mby?

	GAME.Fonts.default = love.graphics.getFont( )
	setmetatable(GAME.Fonts,{__index = GAME.Fonts.default}) --fallback, we've failed!
	
	local fontdir, sizes = "assets/fonts/", {18,24,48}
	--GAME:AddHook("LOAD","LoadFonts",function(arg)
		for k, v in ipairs(lfs.getDirectoryItems(fontdir)) do
			local file = fontdir..v
			if lfs.isFile(file) then
				if file:sub(-4,-1):lower() == ".ttf" then
					local name = v:match("^(.*)%."):gsub(" ","_"):lower()
					GAME.Fonts[name] = love.graphics.newFont( file, 13 )
					for si, size in ipairs(sizes) do
						GAME.Fonts[name.."_"..tostring(size)] = love.graphics.newFont( file, size )
					end
					--print("FONT",v,name)
					--MsgC(suc_col,"Loaded FONT '"..v.."' as '"..name.."'")
				end
			end
		end
		GAME.Fonts.default = GAME.Fonts["anonymous_pro_b"]
	--end)
end

recursiveLoad("ui")
recursiveLoad("gui")