--[[SirRanjid 2020 https://github.com/SirRanjid]]

CurTime = love.timer.getTime
ScrW, ScrH = love.graphics.getWidth, love.graphics.getHeight
function Color(r,g,b,a) --global color function
	return {r,g,b,a or 255}
end

function VarFunc(tbl,name,default)
	tbl[name] = default
	
	tbl["Get"..name] = function(self)
		return self[name]
	end
	
	tbl["Set"..name] = function(self,v)
		self[name] = v
	end
end


local debugmsg = {msg = {}, temp = {}, draw = true}
local hooks = {}

--[[
	Hook functions
--]]

	function GAME:AddHook(hook, name, func)
		hooks[hook] = hooks[hook] or {}
		hooks[hook][name] = func
	end
	
	function GAME:RemoveHook(hook, name)
		if not hooks[hook] then return end
		hooks[hook][name] = nil
	end
	
	function GAME:CallHook(hook,...)
		if not hooks[hook] then return false end
		for k,v in pairs(hooks[hook]) do
			v(...)
		end
	end
	
--[[
	Timer functions
--]]
do
	local timers = {}
	local l_timer = {}
	
	function GAME:StopTimer( name )
		if l_timer[name] then
			for i,v in ipairs(timers) do
				if v[2] == name then table.remove(timers,i) break end
			end
			l_timer[name] = nil
		end
	end
	
	function GAME:Timer(name,sec,func,...)
		GAME:StopTimer( name )
		local T = CurTime() + sec
		local new = {T,name,func,{...}}
		l_timer[name] = new
		local cT = #timers
		if cT > 0 then
			for I = 1, cT do
				local tbl = timers[I]
				if tbl[1] > T then
					table.insert(timers,I,new)
				elseif I == cT then
					table.insert(timers,new)
				end
			end
		else
			table.insert(timers,new)
		end
	end
	
	local CurTime = CurTime
	GAME:AddHook("Tick","Timers",function(dt)
		if timers[1] then
			local T = CurTime()
			
			for i,v in ipairs(timers) do
				if v[1] <= T then
					GAME:StopTimer( v[2] )
					if v[4] then
						v[3](unpack(v[4]))
					else
						v[3]()
					end
				else
					break
				end
			end
		end
	end)
end
--[[
	Debug functions
--]]
function GAME:SetDrawDebug(tf)
	debugmsg.draw = tf
end

function GAME:AddDebug(txt,tme,col) --disappearing, unsorted debugtext
	if col then
		debugmsg.temp[#debugmsg.temp + 1] = {{col,txt},CurTime() + tme}
	else
		debugmsg.temp[#debugmsg.temp + 1] = {txt,CurTime() + tme}
	end
end

function GAME:AddDebugID(id,txt,tme,col) --persisting debugmsg text (tme = 0: infinite)
	local ti = (tme == 0) and 0 or (CurTime() + tme)
	if col then
		debugmsg.msg[id] = {{col,txt},ti}
	else
		debugmsg.msg[id] = {txt,ti}
	end
end

local drawtext = love.graphics.print
local setcolor = love.graphics.setColor
local cw = Color(255,255,255)
GAME:AddHook("DrawDebug","DrawDebug",function(dt)
	if debugmsg.draw then
		local c,t = 0, CurTime()
		setcolor(cw)
		for k,v in pairs( debugmsg.msg ) do
			if v[2] and (v[2] == 0 or v[2] > t) then
				drawtext(v[1], 0, c*14)
			end
			c = c+1
		end
		
		for i,v in ipairs( debugmsg.temp ) do
			if v[2] > t then
				drawtext(v[1], 10, 20+(c+i)*14)
			else
				table.remove(debugmsg.temp,i)
			end
		end
	end
end)