--[[SirRanjid 2020 https://github.com/SirRanjid]]

local Slider = UIClass("Slider",UI.Base)
UI.Slider = Slider

local bg_col, hov_col, t_col = 
Color(220,220,220,180), Color(170,220,255,220), Color(0,0,0)


local a_col = Color(50,160,255,120)
VarFunc(Slider,"Text","Slider")
VarFunc(Slider,"Suffix","")
VarFunc(Slider,"Font",GAME.Fonts["vollkornsc-regular"])

VarFunc(Slider,"Min",0)		--minimum value
VarFunc(Slider,"Max",100)	--maximum value
VarFunc(Slider,"Value",100)	--actual value
VarFunc(Slider,"Steps",1)	--stepsize
VarFunc(Slider,"Mod",0.01)		--value output multiplier

VarFunc(Slider,"TextColor",t_col)
VarFunc(Slider,"Color",a_col)
VarFunc(Slider,"HColor",hov_col)
VarFunc(Slider,"BGColor",bg_col)




function Slider:init()
	self.cpt = false
	self.pcttext = self.Text..": "..self.Value.."/"..self.Max.." "..self.Suffix
end
function Slider:SetText(txt)
	self.Text = txt
	self.pcttext = txt..": "..self.Value.."/"..self.Max.." "..self.Suffix
end

function Slider:GetValue()
	return self.Value * self.Mod
end

function Slider:SetValue(val)
	self.Value = val
	self.pcttext = self.Text..": "..self.Value.."/"..self.Max.." "..self.Suffix
	self:OnNewValue(self:GetValue())
end
function Slider:SetFont(str)
	self.Font = GAME.Fonts[str] or GAME.Fonts["vollkornsc-regular_18"]
end

local rect = love.graphics.rectangle
local drawtext = love.graphics.print
local setfont = love.graphics.setFont
local setcolor,getcolor = love.graphics.setColor, love.graphics.getColor
local align = "center"
local mfill, mline = "fill","line"
local pct = 0

function Slider:OnLeftClick()
	self.cpt = true
end

local isDown = love.mouse.isDown
function Slider:Draw(x,y,w,h)
	if self.Font ~= "" then setfont(self.Font) end
	
	local tw,th = self.Font:getWidth(self.pcttext), self.Font:getHeight(self.pcttext)
	
	setcolor(self.BGColor)
	rect(mfill,x,y,w,h)
	
	if self.cpt then
		setcolor(self.HColor)
		
		local pct = (self.mx)/(w-4)
		local nval = self.Min+(self.Max-self.Min)*pct --
		nval = math.max(self.Min,math.min(self.Max,nval))
		nval = math.floor(nval/self.Steps) * self.Steps
		
		self:SetValue(nval)
		
		if not isDown(1) then self.cpt = false end
		
	elseif self.hover then
		setcolor(self.HColor)
	else
		setcolor(self.Color)
	end
	
	pct = (self.Value-self.Min)/(self.Max-self.Min)
	
	rect(mline,x+2,y+2,(w-4),h-4)
	rect(mfill,x+2,y+2,(w-4)*pct,h-4)
	
	setcolor(self.TextColor)
	drawtext(self.pcttext,x+(w-tw)/2,y+(h-th)/2)

end

function Slider:OnNewValue(nval) end