--[[SirRanjid 2020 https://github.com/SirRanjid]]

GAME = {}
GAME.__index = GAME

require("_util")
require("_input_hooks")

local physics = love.physics

local physmeter = 8
physics.setMeter(physmeter)
world = physics.newWorld(0, 0, true)

local ScrW, ScrH = love.graphics.getWidth, love.graphics.getHeight
local physics = love.physics

local color = love.graphics.setColor
local circ = love.graphics.circle
local line = love.graphics.line

local isMDown,isDown = love.mouse.isDown,love.keyboard.isDown

local m1d = false
local m2d = false

local dpx, dpy = 0,0
local drx, dry = 0,0

local sin,cos,atan2,pi,flr = math.sin,math.cos,math.atan2,math.pi,math.floor

spdm = 1		--sim speed multiplier
learn_rate = 0.01	--learn rate/color change speed
unlearn_rate = 1	--forget rate/color reverse speed
GC = 80				--gravit multiplier
damp  = 50			--drag force
range = 20			--selection radius
acceptance = -1	--tolerance
connt = 0.6			--remember/link decay

rest = 0
fric = 0
dforce = 10 --distm

subt=4
local ast = 1/subt

cblobs = 69

local blobs = {}


local cvs = love.graphics.newCanvas(ScrW(),ScrH(),"rgba32f")
local cv2 = love.graphics.newCanvas(ScrW(),ScrH(),"rgba32f")
local cv3 = love.graphics.newCanvas(ScrW(),ScrH(),"rgba32f")

math.randomseed(os.time())

local _fill,_line = "fill","line"

function rot(x,y,r)
	return x*cos(r*pi/180),y*sin(r*pi/180)
end

function hsv( h, s, v, a )	--https://github.com/EmmanuelOga/columns/blob/master/utils/color.lua (LICENSE: http://creativecommons.org/licenses/by/3.0/legalcode)
	local h=h/360

	local i = flr(h * 6);
	local f = h * 6 - i;
	local p = v * (1 - s);
	local q = v * (1 - f * s);
	local t = v * (1 - (1 - f) * s);

	local i = i % 6

	if i == 0 then r, g, b = v, t, p
	elseif i == 1 then r, g, b = q, v, p
	elseif i == 2 then r, g, b = p, v, t
	elseif i == 3 then r, g, b = p, q, v
	elseif i == 4 then r, g, b = t, p, v
	elseif i == 5 then r, g, b = v, p, q
	end
	
	return {r* 255, g* 255, b* 255, (a or 1) * 255}
end


function newp(px,py,sx,sy,r,m,cv,dyn)	--new planet (pos x, pos y, vel x, vel y, radius, mass, hue, dynamic? or static)
	--m = m * 1000
	local new = {
		body = physics.newBody(world, px, py,"dynamic"), 
		shape = physics.newCircleShape(r),
		cvs = love.graphics.newCanvas(r*2,r*2,"rgba32f"),
		cv2 = love.graphics.newCanvas(r*2,r*2,"rgba32f"),
		col = hsv(cv,1,0.9), --color
		acol = hsv(cv,1,0.9),
		cv = cv,	--hue
		vol = r*math.pi^3, --volume
		tm = m, --total mass
		fx=0,
		fy=0,
		dyn = dyn,
		r=r,
		br = {},
		cont=0,
	}

	function new:updateprops()
		local ma = 100+self.acol[2] /2.5
		self.fixture:setDensity( ma/self.vol )
		self.tm=ma
		self.body:setMass( ma )
		self.body:setInertia( ma )

		self.fixture:setRestitution( (self.acol[1]/255)*2 ) 
		self.fixture:setFriction( self.acol[3]/255 )
	end

	new.body:setUserData(new)

	new.body:isBullet(true)
	new.fixture = physics.newFixture(new.body, new.shape)
	new.fixture:setRestitution( rest ) 
	new.fixture:setFriction( fric )
	new.m = new.tm/new.vol
	new.fixture:setDensity( new.m )

	new.body:setMass( m )
	new.body:setInertia( m )


	new.body:setUserData(new)
	new.fixture:setUserData(new)

	function new:setDynamic(tf)
		self.body:setType(tf and "dynamic" or "static")
		self.body:setMass( self.tm )
		self.body:setInertia( self.m )
	end

	new:setDynamic(dyn)

	new.body:setLinearVelocity( sx,sy )

	return new
end

local colltbl = {}

local sp = 290
for i = 1, cblobs do
	local bl = newp(math.random(1,ScrW()),math.random(1,ScrH()),math.random(-sp,sp),math.random(-sp,sp),16,100,i*6,1)

	bl:updateprops()

	local blob = {0,0,bl = bl}

	colltbl[bl] = {1}
	blobs[#blobs+1]=blob
end

local time  = love.timer.getTime

function clamp(a,b,c)
	if a < b then return b end
	if a > c then return c end
	return a
end



local function cbc(ai,bi,dt)

	local ac = ai.acol
	local bc = bi.acol
	local ac2 = ai.acol
	local bc2 = bi.acol

	if colltbl[ai][bi] then
		dt = dt * colltbl[ai][bi][4]
	end

	local rat = (0.025/colltbl[ai][1])*dt

	ac[1] =  clamp(ac[1] - (ac[1]-bc2[1]) * learn_rate * rat,0,255)
	ac[2] =  clamp(ac[2] - (ac[2]-bc2[2]) * learn_rate * rat,0,255)
	ac[3] =  clamp(ac[3] - (ac[3]-bc2[3]) * learn_rate * rat,0,255)

	local rat = 0.5/colltbl[bi][1]

	bc[1] =  clamp(bc[1] - (bc[1]-ac2[1]) * learn_rate * rat,0,255)
	bc[2] =  clamp(bc[2] - (bc[2]-ac2[2]) * learn_rate * rat,0,255)
	bc[3] =  clamp(bc[3] - (bc[3]-ac2[3]) * learn_rate * rat,0,255)

	ai:updateprops()
	bi:updateprops()

end

local lmd,md  = false,false
do
	local tScrW, tScrH, cScrW, cScrW = ScrW(), ScrH(), ScrW(), ScrH()
	
	local getMode = love.window.getMode
	local tX,tY,cX,cY,flags = 0,0,0,0, {}

	function love.update(dt)
		dt=clamp(dt,0,0.1)*spdm
		local mx,my = love.mouse.getPosition()

		local blc = #blobs

		GAME:CallHook("Tick",dt)

		cScrW, cScrH, flags = getMode()
		if tScrW ~= cScrW or tScrH ~= cScrH then
			local dW, dH = cScrW-tScrW, cScrH-tScrH
			GAME:CallHook("UpdateResolution", cScrW, cScrH, dW, dH, cScrW/2, cScrH/2)
			tScrW, tScrH = cScrW, cScrH
		end

		cX,cY = flags.x, flags.y
		if tX ~= cX or tY ~= cY then
			GAME:CallHook("WindowMoved", cX, cY, tX, tY)
			tX, tY = cX, cY
		end

		lmd = md
		md = isMDown(3)
		local m2 = isMDown(2)

		for I = 1, subt do --5 times per tick for more accuracy
			for i,v in ipairs(blobs) do
				color(v.bl.col)
				local x, y = v.bl.body:getX(), v.bl.body:getY()
				if x < 1 then
					v.bl.body:setX(ScrW())
				elseif x > ScrW() then
					v.bl.body:setX(1)
				end

				if y < 1 then
					v.bl.body:setY(ScrH())
				elseif y > ScrH() then
					v.bl.body:setY(1)
				end

				local vx,vy = v.bl.body:getLinearVelocity()
				v.bl.body:applyForce(-vx*damp*v.bl.tm*ast,-vy*v.bl.tm*damp*ast)

				for j,v2 in ipairs(blobs) do --calculate gravitational force between planets
					if v.bl ~= v2.bl then
						
						if colltbl[v.bl][v2.bl] and colltbl[v.bl][v2.bl][2]+(connt/spdm)-time()>0 then cbc(v.bl,v2.bl,dt*ast) end 

						local dx = v2.bl.body:getX()-v.bl.body:getX()
						local dy = v2.bl.body:getY()-v.bl.body:getY()

						if dx > ScrW()/2 then
							dx = ScrW()/2 - dx
						end

						if dy > ScrH()/2 then
							dy = ScrH()/2 - dy
						end

						local d = dx^2+dy^2
						local mm = v2.bl.tm / d

						if v.bl.bcol then
							mm = 0
						elseif v2.bl.bcol then
							mm = mm * 4
						end
							
						local ac = v.bl.acol
						local bc = v2.bl.acol

						local rang = acceptance > 0 and acceptance^0.5 or -((acceptance^2)^0.25)

						local len = 1 - (((ac[1]-bc[1])/128)^2+ ((ac[2]-bc[2])/128)^2+ ((ac[3]-bc[3])/128)^2)*(1-rang)
						local ang = atan2(dy,dx)

						local fx	= cos(ang) * GC * mm * len * 64000
						local fy	= sin(ang) * GC * mm * len * 64000

						if j == 1 then
							v.bl.fx = 0
							v.bl.fy = 0
						end

						v.bl.fx = v.bl.fx + fx*ast
						v.bl.fy = v.bl.fy + fy*ast
						v.bl.body:applyForce( fx*ast,fy*ast )
								
					end
				end

				circ("fill", v.bl.body:getX(), v.bl.body:getY(), v.bl.shape:getRadius())
				
				local ac, bc = v.bl.acol, v.bl.col

				do
					local umul = (3/(1+v.bl.cont)) * unlearn_rate*dt*ast

					v.bl.acol[1] = clamp(ac[1] - (ac[1] - bc[1]) * umul,0,255)
					v.bl.acol[2] = clamp(ac[2] - (ac[2] - bc[2]) * umul,0,255)
					v.bl.acol[3] = clamp(ac[3] - (ac[3] - bc[3]) * umul,0,255)
				end

				v.bl:updateprops()

				local ran = ((((mx-x)^2)^0.5) < range and (((my-y)^2)^0.5) < range)
				if ran or v.bl.bcol or (m2 and ran) then
					if md and md ~= lmd then
						if m2 then 
						v.bl.bcol = {255-v.bl.col[1],255-v.bl.col[2],255-v.bl.col[3],255}
						else
						v.bl.bcol = {255-v.bl.acol[1],255-v.bl.acol[2],255-v.bl.acol[3],255}
						end
					end

					if (md and v.bl.bcol) or m2 then
						if m2 then 
							v.bl.acol = {255-v.bl.col[1],255-v.bl.col[2],255-v.bl.col[3],255}
							v.bl:updateprops()
						end

						if not m2 then
							local dx = (mx-v.bl.body:getX())*(1+damp)*v.bl.tm
							local dy = (my-v.bl.body:getY())*(1+damp)*v.bl.tm

							if ((mx-v.bl.body:getX())^2)^0.5 > ScrW()/2 then
								dx = -dx
							end

							if ((my-v.bl.body:getY())^2)^0.5 > ScrH()/2 then
								dy = -dy
							end
						
							local vx,vy = v.bl.body:getLinearVelocity()
							v.bl.body:applyForce( (dx-vx*2)*ast*v.bl.tm,(dy-vy*2)*ast*v.bl.tm )
						end

					end
				end

				if not (md or m2) and v.bl.bcol then
					v.bl.bcol = nil
				end

			end

			world:update(dt/subt)
		end
	end
end


local color = love.graphics.setColor
local circ = love.graphics.circle
local line = love.graphics.line
local clear,setcanvas,blendmode,draw = love.graphics.clear, love.graphics.setCanvas, love.graphics.setBlendMode, love.graphics.draw
local setshader = love.graphics.setShader

local lt = time()
local getDT = love.timer.getDelta

local bvs,gvs
function love.draw()
	local mx,my = love.mouse.getPosition()
	local ft = time() - lt
	lt = time()

	local scrw2,scrh2 = ScrW()/2,ScrH()/2 

	setcanvas(cvs) 
		clear(0,0,0,255)
	setcanvas()
	setcanvas(cv2) 
		clear(0,0,0,255)
	setcanvas()  
	setcanvas(cv3) 
		clear(0,0,0,255)
	setcanvas()  

	if not bvs then
		bvs = love.graphics.newCanvas(64,64,"rgba32f")

		setcanvas(bvs) 
			clear(0,0,0,255)
			blendmode( "alpha" )
			for i = 1,16 do
				color(255,255,255,256/(i^2))
				circ("fill", 32, 32, 16+i)
			end

			blendmode( "subtract" )
			local stps = 16
			for i = 2,16 do
				local dsp = math.cos(math.pi*((i-1)/stps)*0.5)*92

				color(dsp,dsp,dsp,255)
				circ("fill", 32, 32, 17-i)
			end

		setcanvas()
	end

	for k=1,cblobs do
		local v = blobs[k]
		local bl = v.bl
		
		bl.cont = 0

		setcanvas(cvs) 
		blendmode( "replace" )

		for i=k+1,cblobs do
			local v2 = blobs[i]
			local bl2 = v2.bl
			local ct = colltbl[bl][bl2]

			if ct then
				local tv = 1
				if not ct[1] then
					tv = clamp(ct[2]+(connt/spdm)-time(),0,1)
				else
					ct[2] = time()
				end

				local rtv = tv/(connt/spdm)

				if tv > 0 then
					
					colltbl[bl][bl2][4] = rtv
					colltbl[bl2][bl][4] = rtv
					bl.cont = bl.cont+(rtv)^0.5
					bl2.cont = bl2.cont+(rtv)^0.5

					local p1x,p1y,p2x,p2y = bl.body:getX()-v[1], bl.body:getY()-v[2], bl2.body:getX()-v2[1], bl2.body:getY()-v2[2]
					local dx,dy = ((p1x-p2x)^2)^0.5,((p1y-p2y)^2)^0.5
					
					local dis = 1-clamp(((dx+dy)/800)^0.5,0,0.95)
					color(	(bl.col[1]+bl2.col[1])/2,
						(bl.col[2]+bl2.col[2])/2,
						(bl.col[3]+bl2.col[3])/2,
						255*(tv*dis))
					
					if dx > scrw2 then
						if p1x < p2x then
							line(p1x+ScrW(),p1y,p2x,p2y)
							line(p1x,p1y,p2x-ScrW(),p2y)
						else
							line(p1x-ScrW(),p1y,p2x,p2y)
							line(p1x,p1y,p2x+ScrW(),p2y)
						end
					elseif dy > scrh2 then
						if p1y < p2y then
							line(p1x,p1y+ScrH(),p2x,p2y)
							line(p1x,p1y,p2x,p2y-ScrH())
						else
							line(p1x,p1y-ScrH(),p2x,p2y)
							line(p1x,p1y,p2x,p2y+ScrH())
						end
					else
						
						line(p1x,p1y,p2x,p2y)
					end
				elseif colltbl[bl][bl2][4] > 0 then
					colltbl[bl][bl2][4] = 0
					colltbl[bl2][bl][4] = 0
				end
			end
		end
		setcanvas() 
		
		setcanvas(cv2)

			blendmode( "add" )
			color(bl.acol[1],bl.acol[2],bl.acol[3],8)
			
			blendmode( "alpha" )
			circ("fill", bl.body:getX(), bl.body:getY(), bl.shape:getRadius())
			color(bl.col)

			blendmode( "add" )
			color(bl.acol[1],bl.acol[2],bl.acol[3],8+8*bl.cont)
			draw(bvs,bl.body:getX()-32, bl.body:getY()-32)

			blendmode( "alpha" )
			
			
			if ((((mx-bl.body:getX())^2)^0.5) < range and (((my-bl.body:getY())^2)^0.5) < range) or bl.bcol then
				circ("fill", bl.body:getX(), bl.body:getY(), bl.shape:getRadius()*0.6)
			else

				local x,y=bl.body:getX(), bl.body:getY()
				local vx,vy = bl.body:getLinearVelocity()

				local len = clamp((vx^2+vy^2)^0.5/8,-10,10)
				local ang = atan2(-vy,-vx)

				local vx	= cos(ang) * len
				local vy	= sin(ang) * len

				local tm = ft*21
				local tm2 = ft*13

				v[1] = v[1] - (clamp(v[1]-vx,-tm,tm)  - clamp(v[1],-tm2,tm2))*subt
				v[2] = v[2] - (clamp(v[2]-vy,-tm,tm)  - clamp(v[2],-tm2,tm2))*subt
				blendmode( "add" )
				local px,py,ra = x-v[1], y-v[2], bl.shape:getRadius()/6
				
				--circ("fill", px,py,ra)

				setcanvas(bl.cv2)
					clear(0,0,0,0)
					color(0,0,0,255)
					blendmode( "replace" )
					circ("fill", bl.r,bl.r,bl.r)
					blendmode( "add" )
					color(255,255,255,112)
					draw(bl.cvs,vx/7,vy/7)
					draw(bl.cvs)
				setcanvas()

				setcanvas(bl.cvs)
					clear(0,0,0,255)
					color(255,255,255,245)
					draw(bl.cv2)
					color(bl.col[1],bl.col[2],bl.col[3],255)
					circ("fill", bl.r-v[1],bl.r-v[2],ra+ra*bl.cont*0.3)
				setcanvas()

				setcanvas(cv2)
					color(255,255,255,80)
					draw(bl.cv2,x-bl.r,y-bl.r)

					color(bl.col[1],bl.col[2],bl.col[3],50)
					circ("fill", px,py,ra)
					color(bl.col[1],bl.col[2],bl.col[3],255)
					circ("fill", px,py,ra/2)
				setcanvas()
			end
		setcanvas()
		local txt = bl.acol[1] .. ", " .. bl.acol[2] .. ", " .. bl.acol[3]
		local sx = love.graphics.getFont( ):getWidth( txt )/2
	end
	setcanvas(cv3)
		clear(0,0,0,0)
		color(255,255,255,255)
		blendmode("alpha")
		draw(cv2)
		blendmode("multiply")
		draw(cvs)
		
	setcanvas()

	blendmode("alpha")
	color(255,255,255,255)
	draw(cv2)
	blendmode("add")
	draw(cv3)
	draw(cv3,1,1)
	draw(cv3,1,-1)
	draw(cv3,-1,1)
	draw(cv3,-1,-1)
	draw(cv3)
	draw(cv3)

	blendmode("alpha")

	local dt = getDT()
	GAME:CallHook("Draw",dt)
	--GAME:CallHook("DrawDebug",dt)

end


local sltbl = {
	{"learn_rate","Learn Rate",0,1,0.01},
	{"unlearn_rate","Forget Rate",0,5,0.05},
	{"acceptance","Tolerance",-1,1,0.01},
	{"GC","Gravity",-300,300,10},
	{"damp","Drag",0,500,10},
	{"connt","Remember",0,5,0.1,"sec"},
	{"spdm","Sim Speed",0,5,0.025},
	{"range","Selection Radius",20,200,5},
	{"subt","PhysSteps/Tick",3,10,1},
}


function love.load()
	require("_loader")

	local MainMenu	=	UI("StackP",0,0,ScrW(),ScrH())
	local slr

	for i,v in ipairs(sltbl) do

		slr = MainMenu:Add("Slider",20,ScrH()-(25*i),420,20)
		slr:SetText(v[2])
		slr:SetMin(v[3])
		slr:SetMax(v[4])
		if v[6] then slr:SetSuffix(v[6]) end
		slr:SetMod(1)
		slr:SetSteps(v[5])
		slr:SetValue(_G[v[1]])

		slr:SetTextColor(Color(0,120,255,80))
		slr:SetColor(Color(50,50,50,10))
		slr:SetHColor(Color(60,150,220,30))
		slr:SetBGColor(Color(0,0,0,50))

		slr:Align(true,0,1,false,false,20,-25*i,0,0)

		function slr:OnNewValue(nval)
			_G[v[1]] = nval
		end
	end
	slr:SetValue(subt)
	function slr:OnNewValue(nval)
			
		subt = nval
		ast = 1/subt
	end


	MainMenu:SetColor(Color(0,0,0,0))
	MainMenu:Open(true)


	world:setCallbacks( function(a,b,c) 
		if not colltbl[a:getUserData()] then colltbl[a:getUserData()] = {1,true} end 
		if not colltbl[b:getUserData()] then colltbl[b:getUserData()] = {1,true} end

		colltbl[a:getUserData()][b:getUserData()] = {true,time(),true,0}
		colltbl[b:getUserData()][a:getUserData()] = {true,time(),true,0}
		colltbl[a:getUserData()][1] = colltbl[a:getUserData()][1]+1
		colltbl[b:getUserData()][1] = colltbl[b:getUserData()][1]+1
	end,
	function(a,b,c)
		colltbl[a:getUserData()][b:getUserData()][1] = false
		colltbl[b:getUserData()][a:getUserData()][1] = false
		colltbl[a:getUserData()][1] = colltbl[a:getUserData()][1] - 1
		colltbl[b:getUserData()][1] = colltbl[b:getUserData()][1] - 1
	end )
end